/**
 * Copyright 2025 the WHATAP project authors. All rights reserved.
 * Use of this source code is governed by a license that
 * can be found in the LICENSE file.
 */

var CounterTask     = require('./counter-task'),
    DataPackSender  = require('../../data/datapack-sender'),
    conf = require('../../conf/configure'),
    DateUtil = require('../../util/dateutil'),
    Logger = require('../../logger'),
    fs = require('fs'),
    TagCountPack = require('../../pack/tagcount-pack'),
    OType = require('../../pack/otype'),
    APEnum = require('../../pack/apenum')

function MeteringInfo() {
    CounterTask.call(this);
    this.first_connected = true;
    this.lastTime = DateUtil.getFiveMinUnit(DateUtil.currentTime());
    this.hostUUID = null;
    this.csp = null;
    this.hostUUIDFileName = '/sys/class/dmi/id/product_uuid';
}

MeteringInfo.prototype = new CounterTask();
MeteringInfo.prototype.constructor = MeteringInfo;

MeteringInfo.prototype.process = function (p) {
    if (!conf.getProperty('metering_tagcount_enabled', false)) {
        return;
    }

    if (this.first_connected) {
        this.first_connected = false;
        this.doProcess(p);
        return;
    }

    const now = DateUtil.getFiveMinUnit(DateUtil.currentTime());
    if (this.lastTime >= now) {
        return;
    }

    this.lastTime = now;
    this.doProcess(p);
};

MeteringInfo.prototype.doProcess = function (p) {
    const pk = new TagCountPack();
    pk.category = 'metering';
    pk.time = DateUtil.currentTime() / 300000 * 300000;

    pk.putTagInt('otype', OType.AP);
    pk.putTagInt('subtype', APEnum.AP_NODE);
    pk.putTagInt('ip', p.host_ip);

    const host_uuid = this.getHostUUID(this.hostUUIDFileName);
    if (host_uuid) {
        pk.putTagString('host_uuid', host_uuid);
    }

    const csp = this.getCsp();
    if (csp) {
        pk.putTagString('csp', csp)
    }

    pk.putTagFloat(p.metering)

    DataPackSender.sendTagCounterPack(pk);
};

MeteringInfo.prototype.hasHostUUID = function (fileName) {
    try{
        return fs.existsSync(fileName);
    }catch (e) {
        Logger.printError("WHATAP-081", "hasHostUUID error", e, false);
        return false;
    }
};

MeteringInfo.prototype.getHostUUID = function (fileName) {
    const hostUUID = this.getHostUUIDValue();
    if (hostUUID) {
        return hostUUID;
    }

    const hasHostUUIDFile = this.hasHostUUID(fileName);
    if (hasHostUUIDFile) {
        try {
            const data = fs.readFileSync(fileName);

            if (data && data.length > 0) {
                this.hostUUID = data.toString().trim();
                return this.hostUUID;
            } else {
                Logger.print('WHATAP-082', 'cannot read /sys/class/dmi/id/product_uuid', false);
            }
        } catch (error) {
            Logger.printError('WHATAP-083', 'error reading /sys/class/dmi/id/product_uuid ', error, false);
        }
    } else {
        Logger.print('WHATAP-084', 'cannot find /sys/class/dmi/id/product_uuid', false);
    }

    return null;
};

MeteringInfo.prototype.getHostUUIDValue = function () {
    return this.hostUUID;
};

MeteringInfo.prototype.getCsp = function () {

    return null;
};

module.exports = MeteringInfo;